"""Configuration values, access, and management

The default configuration targets a local, single-node instance of Vespa with
all standard ports and configuration.
"""

import os


#------------------------------------------------------------------------------
# Configuration
#------------------------------------------------------------------------------

# Vespa
vespa_host_cfg = "http://127.0.0.1:19071"
vespa_host_app = "http://127.0.0.1:8080"

# Venra
baz = True


#------------------------------------------------------------------------------
# Config management
#------------------------------------------------------------------------------

def load_overrides_from_env():
    """Load any override configuration values from env vars

    example:
    > export VESPA_HOST="192.168.1.50"
    """

    if "VESPA_HOST_CFG" in os.environ:
        global vespa_host_cfg
        vespa_host_cfg = os.environ["VESPA_HOST_CFG"]

    if "VESPA_HOST_APP" in os.environ:
        global vespa_host_app
        vespa_host_app = os.environ["VESPA_HOST_APP"]

    return


def load_overrides_from_file():
    """Load any override configuration values from config file

    """
    pass




"""Vespa client

"""

import requests

from . import config


vclient = None


def reset():
    """Remove initialized client"""
    global vclient
    vclient = None


def get_vespa_client():
    """Get or create a vespa client and return"""

    global vclient

    # initialize session if not already done
    if vclient is None:
        vclient = requests.Session()

    return vclient






"""Vespa visit api

"""


from . import client
from . import config


def feed(namespace, doctype, selection=None):
    """Yield all docs of a given type and selection

    """
    visit_base_uri = f"{config.vespa_host_app}/document/v1/{namespace}/{doctype}/docid"

    # initial call
    vc = client.get_vespa_client()
    if selection:
        vr = vc.get(f"{visit_base_uri}?selection={selection}").json()
    else:
        vr = vc.get(f"{visit_base_uri}").json()

    # page through results
    while True:
        for d in vr.get("documents", []):
            yield d
        continuation = vr.get("continuation", False)
        if continuation:
            vr = vc.get(f"{visit_base_uri}?continuation={continuation}").json()
        else:
            break

    return




"""Vespa document api

"""

import json

from . import client
from . import config



def partial_update(namespace, doctype, docid, operations):
    """Apply a partial update using the assign, add, or remove operator

    operations = [("assign", "field_name_here", "field_value_here"), ...]

    reference:
    * https://docs.vespa.ai/en/reference/document-json-format.html

    Example:
    * https://docs.vespa.ai/en/reference/document-json-format.html#assign-map-field
    {
        "update": "id:mynamespace:food::example",
        "fields": {
            "my_food_scores{Strawberries}": { "assign": "Delicious!" }
        }
    }
    """
    base_uri = f"{config.vespa_host_app}/document/v1/{namespace}/{doctype}/docid/{docid}"
    partial_update = {"fields": {}}
    for operation, field_name, field_value in operations:
        partial_update["fields"][field_name] = {operation: field_value}

    vc = client.get_vespa_client()
    vr = vc.put(f"{base_uri}", data=json.dumps(partial_update))

    return vr





from setuptools import setup, find_packages

exec(open('venra/version.py').read())

with open("README.md", mode="r", encoding="utf-8") as f:
    readme_contents = f.read()


setup(
  name = 'venra',
  packages = find_packages(exclude=[]),
  version = __version__,
  license='MIT',
  description = 'venra',
  author = 'Cody Collier',
  author_email = 'cody@telnet.org',
  long_description=readme_contents,
  long_description_content_type = 'text/markdown',
  url = 'https://github.com/codycollier/venra',
  keywords = [
    'artificial intelligence',
    'information retrieval',
    'machine learning',
    'search',
  ],
  python_requires=">=3.7, <4",
  install_requires=[
    'numpy',
    'packaging',
    'requests',
    'torch',
    'transformers',
  ],
  classifiers=[
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'Topic :: Scientific/Engineering :: Artificial Intelligence',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
    'Programming Language :: Python :: 3.10',
  ],
)



"""Vespa system apis

"""


from . import client
from . import config


def version_app():
    """Return the version string from the app server

    example:
    $ curl -s http://127.0.0.1:8080/state/v1/version | jq .
    {
        "version": "8.13.21"
    }
    """
    vc = client.get_vespa_client()
    vr = vc.get(f"{config.vespa_host_app}/state/v1/version").json()
    return vr["version"]


def version_cfg():
    """Return the version string from the app server

    example:
    $ curl -s http://127.0.0.1:19071/state/v1/version | jq .
    {
        "version": "8.13.21"
    }
    """
    vc = client.get_vespa_client()
    vr = vc.get(f"{config.vespa_host_cfg}/state/v1/version").json()
    return vr["version"]


